# Generated by Django 3.2.4 on 2021-10-08 15:03

import json
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


def convert_variables_to_reduction_arguments(apps, _):
    ReductionRun = apps.get_model("reduction_viewer", "ReductionRun")
    ReductionArguments = apps.get_model("reduction_viewer", "ReductionArguments")

    for run in ReductionRun.objects.all():
        arguments_dict = {}
        arguments_dict["standard_vars"] = {}
        arguments_dict["advanced_vars"] = {}
        arguments_dict["variable_help"] = {
            "standard_vars": {},
            "advanced_vars": {},
        }
        for run_variable in run.run_variables.all():
            var = run_variable.variable
            if not var.is_advanced:
                key = "standard_vars"
            else:
                key = "advanced_vars"
            arguments_dict[key][var.name] = var.value
            arguments_dict["variable_help"][key][var.name] = var.help_text

        arguments_json = json.dumps(arguments_dict, separators=(',', ':'))
        arguments, created = ReductionArguments.objects.get_or_create(raw=arguments_json,
                                                                      start_run=None,
                                                                      experiment_reference=None,
                                                                      instrument=run.instrument)
        run.arguments = arguments
        run.save()


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0010_move_script_into_reductionscript'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReductionArguments',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('raw', models.TextField(validators=[django.core.validators.MaxLengthValidator(100000)])),
                ('start_run', models.IntegerField(null=True, blank=True)),
                ('experiment_reference', models.IntegerField(null=True, blank=True)),
                ('instrument', models.ForeignKey('Instrument', on_delete=models.CASCADE, related_name="arguments")),
            ],
        ),
        migrations.AddField(
            model_name='reductionrun',
            name='arguments',
            field=models.ForeignKey(null=True,
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='reduction_runs',
                                    to='reduction_viewer.reductionarguments'),
        ),
        migrations.RunPython(convert_variables_to_reduction_arguments),
        migrations.AlterField(model_name='reductionrun',
                              name='arguments',
                              field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                      related_name='reduction_runs',
                                                      to='reduction_viewer.reductionarguments',
                                                      null=False)),
    ]



# Generated by Django 3.2.4 on 2021-10-08 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0011_move_arguments_into_reductionarguments'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datalocation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='instrument',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='output',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='outputtype',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='reductionarguments',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='reductionlocation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='reductionrun',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='reductionscript',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='runnumber',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='setting',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='software',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='status',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 3.2.4 on 2021-10-18 08:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0014_auto_20211012_0443'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='reductionrun',
            name='retry_run',
        ),
    ]



# Generated by Django 3.1.2 on 2021-02-08 14:39

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Experiment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reference_number', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Instrument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=80)),
                ('is_active', models.BooleanField(default=False)),
                ('is_paused', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.CharField(max_length=255)),
                ('is_active', models.BooleanField(default=True)),
                ('severity', models.CharField(choices=[('i', 'info'), ('w', 'warning'), ('e', 'error')], default='i', max_length=1)),
                ('is_staff_only', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='OutputType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('value', models.CharField(max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='Software',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('version', models.CharField(max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name='Status',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(choices=[('q', 'Queued'), ('p', 'Processing'), ('s', 'Skipped'), ('c', 'Completed'), ('e', 'Error')], max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='ReductionRun',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('run_number', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)])),
                ('run_version', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)])),
                ('started_by', models.IntegerField(blank=True, null=True)),
                ('run_name', models.CharField(blank=True, max_length=200)),
                ('admin_log', models.TextField(blank=True)),
                ('graph', models.TextField(blank=True, null=True)),
                ('message', models.TextField(blank=True, null=True)),
                ('reduction_log', models.TextField(blank=True)),
                ('script', models.TextField(validators=[django.core.validators.MaxLengthValidator(100000)])),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('finished', models.DateTimeField(blank=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True)),
                ('retry_when', models.DateTimeField(blank=True, null=True)),
                ('started', models.DateTimeField(blank=True, null=True)),
                ('cancel', models.BooleanField(default=False)),
                ('hidden_in_failviewer', models.BooleanField(default=False)),
                ('overwrite', models.NullBooleanField(default=True)),
                ('experiment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reduction_runs', to='reduction_viewer.experiment')),
                ('instrument', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reduction_runs', to='reduction_viewer.instrument')),
                ('retry_run', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='reduction_viewer.reductionrun')),
                ('software', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reduction_runs', to='reduction_viewer.software')),
                ('status', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='reduction_viewer.status')),
            ],
        ),
        migrations.CreateModel(
            name='ReductionLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_path', models.CharField(max_length=255)),
                ('reduction_run', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reduction_location', to='reduction_viewer.reductionrun')),
            ],
        ),
        migrations.CreateModel(
            name='Output',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_path', models.CharField(max_length=255)),
                ('job', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='output', to='reduction_viewer.reductionrun')),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='output', to='reduction_viewer.outputtype')),
            ],
        ),
        migrations.CreateModel(
            name='DataLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_path', models.CharField(max_length=255)),
                ('reduction_run', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='data_location', to='reduction_viewer.reductionrun')),
            ],
        ),
    ]



# Generated by Django 3.1.2 on 2021-02-24 10:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='reductionrun',
            name='cancel',
        ),
    ]



# Generated by Django 3.2.4 on 2021-08-16 05:30

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


def move_existing_run_numbers(apps, _):
    """Moves the run_number field data into the now related RunNumber model"""
    ReductionRun = apps.get_model("reduction_viewer", "ReductionRun")
    RunNumber = apps.get_model("reduction_viewer", "RunNumber")

    for run in ReductionRun.objects.all():
        RunNumber.objects.create(reduction_run=run, run_number=run.run_number)


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0006_reductionrun_reduction_host'),
    ]

    operations = [
        migrations.CreateModel(
            name='RunNumber',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('run_number', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)])),
                ('reduction_run',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                   related_name='run_numbers',
                                   to='reduction_viewer.reductionrun')),
            ],
        ),
        migrations.RunPython(move_existing_run_numbers),
        migrations.RemoveField(
            model_name='reductionrun',
            name='run_number',
        ),
    ]



# Generated by Django 3.1.2 on 2021-03-16 14:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0003_auto_20210310_1601'),
    ]

    operations = [
        migrations.AddField(
            model_name='instrument',
            name='is_flat_output',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.4 on 2021-10-08 17:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('instrument', '0004_auto_20210426_1600'),
        ('reduction_viewer', '0011_move_arguments_into_reductionarguments'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='runvariable',
            name='reduction_run',
        ),
        migrations.RemoveField(
            model_name='runvariable',
            name='variable',
        ),
        migrations.DeleteModel(name='InstrumentVariable', ),
        migrations.DeleteModel(name='RunVariable', ),
        migrations.DeleteModel(name='Variable', ),
    ]



"""
This migrations moves the reduction script contents from ReductionRun.script into
`ReductionScript.text`. This is done to allow the script to be re-used across runs
if it is not changed.
"""

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


def move_script_into_reduction_script(apps, _):
    ReductionRun = apps.get_model("reduction_viewer", "ReductionRun")
    ReductionScript = apps.get_model("reduction_viewer", "ReductionScript")

    for run in ReductionRun.objects.all():
        try:
            rscript = ReductionScript.objects.get(text=run.tmp_script)
        except:  # pylint:disable=bare-except
            rscript = ReductionScript.objects.create(text=run.tmp_script)
        rscript.save()
        run.script_id = rscript.pk
        run.save()


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0009_reductionrun_run_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReductionScript',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True, validators=[django.core.validators.MaxLengthValidator(100000)])),
            ],
        ),
        migrations.RenameField(model_name='reductionrun', old_name='script', new_name='tmp_script'),
        migrations.AddField(
            model_name='reductionrun',
            name='script',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='reduction_runs',
                to='reduction_viewer.reductionscript',
                # make it null=True first, otherwise there is an IntegrityError,
                # the field will be altered to null=False after the scripts are migrated
                null=True),
        ),
        migrations.RunPython(move_script_into_reduction_script),
        migrations.AlterField(model_name='reductionrun',
                              name='script',
                              field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                      related_name='reduction_runs',
                                                      to='reduction_viewer.reductionscript',
                                                      null=False)),
        migrations.RemoveField(model_name='reductionrun', name='tmp_script'),
    ]



# Generated by Django 3.1.2 on 2021-04-26 14:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('instrument', '0002_auto_20210426_1519'),
        ('instrument', '0002_alter_variable_value'),
    ]

    operations = [
    ]



# Generated by Django 3.2 on 2021-04-22 11:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('instrument', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='variable',
            name='value',
            field=models.CharField(blank=True, max_length=300),
        ),
    ]



# Generated by Django 3.2.4 on 2021-10-12 09:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0013_remove_reductionrun_retry_when'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='output',
            name='job',
        ),
        migrations.RemoveField(
            model_name='output',
            name='type',
        ),
        migrations.DeleteModel(
            name='Setting',
        ),
        migrations.DeleteModel(
            name='Output',
        ),
        migrations.DeleteModel(
            name='OutputType',
        ),
    ]



# ############################################################################### #
# Autoreduction Repository : https://github.com/ISISScientificComputing/autoreduce
#
# Copyright &copy; 2019 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""Models that represent the tables in the database."""
# pylint:disable=no-member
import json
from django.core.validators import MaxLengthValidator, MinValueValidator
from django.db import models


class ReductionScript(models.Model):
    """
    Holds a sharable reduction script's text.

    Fields:
        text: The script's text
    """
    text = models.TextField(blank=True, validators=[MaxLengthValidator(100000)])


class ReductionArguments(models.Model):
    """
    Holds a sharable reduction argument's raw JSON dump representation
    as well as allows for scope of the arguments via start_run and experiment_reference

    Fields:
        start_run: The first run number from which the arguments will be effective.
                They will be passed to the reduction script.
        experiment_reference: The experiment for which the arguments will be effective.
                Experiment arguments override ALL other arguments.
                This is enforced in the queue processor.
    """
    raw = models.TextField(blank=False, validators=[MaxLengthValidator(100000)])
    start_run = models.IntegerField(null=True, blank=True)
    experiment_reference = models.IntegerField(blank=True, null=True)
    instrument = models.ForeignKey('Instrument', on_delete=models.CASCADE, related_name="arguments")

    def as_dict(self) -> dict:
        """Loads the raw string back into a dict object and returns it."""
        return json.loads(self.raw)


class Instrument(models.Model):
    """
    Holds data about an Instrument.

    Fields:
        name: Name of the instrument
        is_active: Whether the instrument is active. If not active,
                   the queue processor will skip runs for it.
        is_paused: Whether the instrument has been MANUALLY paused from the webapp.
        is_flat_output: Determines the output structure of the reduction.
                        If true, the output for all run versions will be in the same folder.
                        If false, each run version will be saved in a separate folder.
    """
    name = models.CharField(max_length=80)
    is_active = models.BooleanField(default=False)
    is_paused = models.BooleanField(default=False)
    is_flat_output = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.name}"

    def get_last_for_rerun(self, runs=None) -> 'ReductionRun':
        """
        Return the last non-skipped run. If all the runs are skipped, return the
        last skipped run.

        Args:
            runs: Pre-queried runs for the instrument.
                  If None, then reduction_runs.all() will be queried from the database.
        Returns:
            The last not-skipped reduction run in the queryset.
        """
        if not runs:
            runs = self.reduction_runs.all()

        last_run = runs.exclude(status=Status.get_skipped()).last()
        if not last_run:
            last_run = runs.last()

        return last_run


class Experiment(models.Model):
    """
    Holds data about an Experiment.

    Fields:
        reference_number: The experiment reference number assigned to the experiment.
                          Also referred to as rb_number because of convention in ISIS.
    """
    reference_number = models.IntegerField(unique=True)

    def __str__(self):
        return f"RB{self.reference_number}"


class Status(models.Model):
    """
    Enum table for status types of messages. Caches the objects internally at runtime
    to avoid repeated queries to the DB for the status objects.

    Fields:
        value: The value of the status. One of STATUS_CHOICES
    """
    _cached_statuses = {}
    STATUS_CHOICES = (('q', 'Queued'), ('p', 'Processing'), ('s', 'Skipped'), ('c', 'Completed'), ('e', 'Error'))

    value = models.CharField(max_length=1, choices=STATUS_CHOICES)

    def value_verbose(self) -> str:
        """Return the status as its textual value."""
        return dict(Status.STATUS_CHOICES)[self.value]

    def __str__(self) -> str:
        return self.value_verbose()

    @staticmethod
    def _get_status(status_value: str):
        """
        Return a status matching the given name or create one if it doesn't yet
        exist.

        Args:
            status_value: The value of the status record in the database.
        """
        if status_value in Status._cached_statuses:
            return Status._cached_statuses[status_value]
        else:
            status_record = Status.objects.get_or_create(value=status_value)[0]
            Status._cached_statuses[status_value] = status_record

        return status_record

    @staticmethod
    def get_error():
        """Return the error status."""
        return Status._get_status('e')

    @staticmethod
    def get_completed():
        """Return the completed status."""
        return Status._get_status('c')

    @staticmethod
    def get_processing():
        """Return the processing status."""
        return Status._get_status('p')

    @staticmethod
    def get_queued():
        """Return the queued status."""
        return Status._get_status('q')

    @staticmethod
    def get_skipped():
        """Return the skipped status."""
        return Status._get_status('s')


class Software(models.Model):
    """Represents the software used to perform the reduction."""
    name = models.CharField(max_length=100, blank=False, null=False)
    version = models.CharField(max_length=20, blank=False, null=False)

    def __str__(self):
        return f"{self.name}-{self.version}"


class ReductionRun(models.Model):
    """
    Table designed to link all table together. This represents a single
    reduction run that takes place at ISIS. Thus, this will store all the
    relevant data regarding that run.


    Fields:
        run_version: The run version of the reduction run.
                     This gets incremented if a run is re-run from the webapp.
        started_by: The user who started the reduction run.
                    Runs submitted from run_detection will have -1 as the user id, which
                    gets assigned "Autoreduction service" in the webapp.
        run_description: The description of the reduction run.
                         Manually specified by the user during re-run.
        run_title: The title of the reduction run. Value is taken from the ICAT
                   entry for the run.
        admin_log: The admin log for the reduction run, containing debug information
                   for the runtime environment.
        graph: FIXME - I am not sure this is used for anything anymore.
        message: The ActiveMQ message with which the reduction run was submitted.
                 Can be used to replicate the reduction run.
        reduction_log: User facing output from the reduction execution, i.e. things that
                       are logged to the stdout/console.
        reduction_host: The hostname of the machine that ran the reduction. Useful
                        for tracking down issues that occurred during reduction.
        created: The date and time the reduction run was created.
        finished: The date and time the reduction run was finished.
        last_updated: The date and time the reduction run was last updated.
        started: The date and time the reduction run was started.

        hidden_in_failviewer: Can be set from failed queue page to hide the run from showing up
                              in the failed queue page again.

        overwrite: If true, the output folder will be overwritten.
        batch_run: Whether this reduction run is for a batch run.

        experiment: Foreign key to the experiment this reduction run belongs to.
        instrument: Foreign key to the instrument this reduction run belongs to.
        arguments: Foreign key to the arguments used to run the reduction.
        script: Foreign key to the script used to run the reduction.
        retry_run: Foreign key to a rerun of this run. Not actively used and
                   will be removed with https://autoreduce.atlassian.net/browse/AR-1554
        status: Foreign key to the status of the reduction run.
        software: Foreign key to the software used to run the reduction.
    """
    # Integer fields
    run_version = models.IntegerField(blank=False, validators=[MinValueValidator(0)])
    started_by = models.IntegerField(null=True, blank=True)

    # Char fields
    run_description = models.CharField(max_length=200, blank=True)
    run_title = models.CharField(max_length=200, blank=True)

    # Text fields
    admin_log = models.TextField(blank=True)
    graph = models.TextField(null=True, blank=True)
    message = models.TextField(null=True, blank=True)
    reduction_log = models.TextField(blank=True)
    reduction_host = models.TextField(default="", blank=True, verbose_name="Reduction hostname")

    # Date time fields
    created = models.DateTimeField(auto_now_add=True, blank=False)
    finished = models.DateTimeField(null=True, blank=True)
    last_updated = models.DateTimeField(auto_now=True, blank=False)
    started = models.DateTimeField(null=True, blank=True)

    # Bool field
    hidden_in_failviewer = models.BooleanField(default=False)
    overwrite = models.NullBooleanField(default=True)
    batch_run = models.BooleanField(default=False)

    # Foreign Keys
    experiment = models.ForeignKey(Experiment, blank=False, related_name='reduction_runs', on_delete=models.CASCADE)
    instrument = models.ForeignKey(Instrument, related_name='reduction_runs', null=True, on_delete=models.CASCADE)
    arguments = models.ForeignKey(ReductionArguments,
                                  blank=False,
                                  related_name='reduction_runs',
                                  on_delete=models.CASCADE)
    script = models.ForeignKey(ReductionScript, blank=False, related_name='reduction_runs', on_delete=models.CASCADE)
    status = models.ForeignKey(Status, blank=False, related_name='+', on_delete=models.CASCADE)
    # Allowed software field to be black in code line below. Issued opened (#852) to later
    # populate this field
    software = models.ForeignKey(
        Software,
        blank=True,
        related_name='reduction_runs',
        null=True,
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return self.title()

    def title(self):
        """
        Return an interface-friendly name that identifies this run using either
        run name or run version.
        """
        try:
            title = f"{self.run_number}"
        except ValueError:
            title = f"Batch {self.run_numbers.first()} → {self.run_numbers.last()}"

        if self.run_version > 0:
            title += f" - {self.run_version}"

        if self.run_title:
            title += f" - {self.run_title}"

        return title

    @property
    def run_number(self) -> int:
        """
        Return the value of the run_number, if only a single one is associated
        with this run. This replicates the behaviour of a one to one
        relationship between a ReductionRun and a RunNumber.
        """
        if self.run_numbers.count() == 1:
            return self.run_numbers.first().run_number
        else:
            raise ValueError(
                "This run has more then one run_number associated with it. You must iterate run_numbers manually")


class RunNumber(models.Model):
    """
    Represents the run number or run numbers that a ReductionRun spans.

    Normal reduction runs will have 1 RunNumber.
    Batch reduction runs will have >1 RunNumbers.


    Fields:
        run_number: A run number associated with the reduction run.
        reduction_run: Foreign key to the reduction run this run number belongs to.
    """
    run_number = models.IntegerField(blank=False, validators=[MinValueValidator(0)])
    reduction_run = models.ForeignKey(ReductionRun, blank=False, related_name='run_numbers', on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.run_number}"


class DataLocation(models.Model):
    """
    Represents the location at which the unreduced data is stored on disk.

    Fields:
        file_path: The path to the file on disk.
        reduction_run: Foreign key to the reduction run this data location belongs to.
    """
    file_path = models.CharField(max_length=255)
    reduction_run = models.ForeignKey(ReductionRun, blank=False, related_name='data_location', on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.file_path}"


class ReductionLocation(models.Model):
    """
    Represents the location at which the reduced data is stored on disk.

    Fields:
        file_path: The path to the file on disk.
        reduction_run: Foreign key to the reduction run this data location belongs to.
    """
    file_path = models.CharField(max_length=255)
    reduction_run = models.ForeignKey(
        ReductionRun,
        blank=False,
        related_name='reduction_location',
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return f"{self.file_path}"


class Notification(models.Model):
    """
    Represents possible notification messages regarding reduction runs.

    Fields:
        message: The message to be displayed.
        is_active: Whether the notification is active or not.
        severity: The severity of the message.
        is_staff_only: Whether the message is only visible to staff.
                       If False, then it will be shown to all users
    """
    SEVERITY_CHOICES = (('i', 'info'), ('w', 'warning'), ('e', 'error'))

    message = models.CharField(max_length=255, blank=False)
    is_active = models.BooleanField(default=True)
    severity = models.CharField(max_length=1, choices=SEVERITY_CHOICES, default='i')
    is_staff_only = models.BooleanField(default=False)

    def __str__(self):
        return f"Notification: {self.message}"

    def severity_verbose(self) -> str:
        """Return the severity as its textual value."""
        return dict(Notification.SEVERITY_CHOICES)[self.severity]



"""
Functionality for project setup and various installations. Enter the following
for more details:
    `python setup.py --help`
"""
# pylint:skip-file
from os import path
from setuptools import setup, find_packages

# Read the contents of the README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='autoreduce_db',
    version='22.0.0.dev17',
    description='ISIS Autoreduce',
    author='ISIS Autoreduction Team',
    url='https://github.com/ISISScientificComputing/autoreduce-db/',
    install_requires=['autoreduce-utils==22.0.0.dev6', 'Django==3.2.8'],
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type='text/markdown',
)



# Generated by Django 3.1.2 on 2021-04-26 15:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('instrument', '0003_merge_20210426_1528'),
    ]

    operations = [
        migrations.AlterField(
            model_name='variable',
            name='value',
            field=models.CharField(blank=True, max_length=300, null=True),
        ),
    ]



# Generated by Django 3.2.4 on 2021-08-19 14:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0007_make_run_number_one_to_many'),
    ]

    operations = [
        migrations.AddField(
            model_name='reductionrun',
            name='batch_run',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.8 on 2021-10-11 11:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0008_reductionrun_batch_run'),
    ]

    operations = [
        migrations.AddField(
            model_name='reductionrun',
            name='run_title',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]



# ############################################################################### #
# Autoreduction Repository : https://github.com/ISISScientificComputing/autoreduce
#
# Copyright &copy; 2020 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""
Remnant file of previously declared Variable models here.
"""
